'use strict';

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _memorystream = require('memorystream');

var _memorystream2 = _interopRequireDefault(_memorystream);

var _babelCodeFrame = require('babel-code-frame');

var _babelCodeFrame2 = _interopRequireDefault(_babelCodeFrame);

var _assert = require('./assert');

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('unfort/utils', () => {
  describe('#createJSModuleDefinition', () => {
    it('should produce a module definition for the bootstrap runtime', () => {
      _assert.assert.equal((0, _utils.createJSModuleDefinition)({
        name: 'test_name',
        hash: 'test_hash',
        deps: { testId: 'test_name' },
        code: 'test_code'
      }), ['__modules.defineModule({name: "test_name", deps: {"testId":"test_name"}, hash: "test_hash", factory: function(module, exports, require, process, global) {', 'test_code', '}});'].join('\n'));
    });
  });
  describe('#createRecordDescription', () => {
    it('should accept a record and produce a description that the hot runtime can interpret', () => {
      _assert.assert.deepEqual((0, _utils.createRecordDescription)({
        name: 'test_name',
        data: {
          hash: 'test_hash',
          url: 'test_url',
          isTextFile: 'test_is_text_file'
        }
      }), {
        name: 'test_name',
        hash: 'test_hash',
        url: 'test_url',
        isTextFile: 'test_is_text_file'
      });
    });
  });
  describe('#describeError', () => {
    it('should accept an error and produce a textual representation for logging', () => {
      const err = new Error('test');
      _assert.assert.equal((0, _utils.describeError)(err), err.stack);
    });
    it('should accept an optional file and provide further context', () => {
      const err = new Error('test');
      _assert.assert.equal((0, _utils.describeError)(err, 'test_filename'), [_chalk2.default.red('test_filename') + '\n', err.stack].join('\n'));
    });
    it('should provide a code frame to further contextualize the error', () => {
      const err = new Error('test');
      err.loc = {
        line: 1,
        column: 1
      };
      _assert.assert.equal((0, _utils.describeError)(err, __filename), [_chalk2.default.red(__filename) + '\n', (0, _babelCodeFrame2.default)(_fs2.default.readFileSync(__filename, 'utf8'), 1, 1), err.stack].join('\n'));
    });
    it('should not override a code frame already attached', () => {
      const err = new Error('test');
      err.codeFrame = 'test code frame';
      _assert.assert.equal((0, _utils.describeError)(err, __filename), [_chalk2.default.red(__filename) + '\n', 'test code frame', err.stack].join('\n'));
    });
    it('should silently fail if a file cannot be read for the code frame', () => {
      const err = new Error('test');
      err.loc = {
        line: 1,
        column: 1
      };
      _assert.assert.equal((0, _utils.describeError)(err, '__file that does not exist__'), [_chalk2.default.red('__file that does not exist__') + '\n', err.stack].join('\n'));
    });
  });
  describe('describeErrorList', () => {
    it('should produce a textual description of a list of errors, such that they can be logged', () => {
      const errors = [new Error('test 1'), new Error('test 2')];

      _assert.assert.equal((0, _utils.describeErrorList)(errors), [(0, _utils.describeError)(errors[0]), (0, _utils.describeError)(errors[1])].join('\n'));
    });
    it('should handle graph error objects (which contain an error and extra data), such that they can be logged', () => {
      const err1 = new Error('test 1');
      const err2 = new Error('test 2');
      const errors = [err1, {
        error: err2,
        node: 'test_filename'
      }];

      _assert.assert.equal((0, _utils.describeErrorList)(errors), [(0, _utils.describeError)(err1), (0, _utils.describeError)(err2, 'test_filename')].join('\n'));
    });
  });
});
//# sourceMappingURL=utils.js.map